@@ -72,7 +72,12 @@ RS_MySQL_init(s_object *config_params, s_object *reload)
   const char *drvName = "MySQL";
   const char *clientVersion = mysql_get_client_info();
 
-  if(strcmp(clientVersion, compiled_mysql_client_version)){
+  /* make sure the versions of the runtime and compile-time of the 
+   * MySQL client library are reasonably close, e.g., first major
+   * and minor id, e.g., 41 for 4.1.*, 42 for 4.2.*, etc.
+   * TODO: Should we using PROTOCOL_VERSION here?
+   */
+  if(strncmp(clientVersion, compiled_mysql_client_version, (size_t) 2)){
      char  buf[256];
      (void) sprintf(buf, 
                     "%s mismatch between compiled version %s and runtime version %s",
@@ -118,17 +123,22 @@ RS_MySQL_cloneConnection(Con_Handle *conHandle)
   Mgr_Handle  *mgrHandle;
   RS_DBI_connection  *con;
   RS_MySQL_conParams *conParams;
-  s_object    *con_params, *MySQLgroups;
+  s_object    *con_params, *MySQLgroups, *s_mysql_default_file;
   char   buf1[256], buf2[256];
 
   /* get connection params used to open existing connection */
   con = RS_DBI_getConnection(conHandle);
   conParams = con->conParams;
 
-  /* will not used the "group" MySQL config file info (use a dummy one) */
+  /* will not used the "group" MySQL config file info, nor the
+   * default_file (use dummy ones) 
+   */
   MEM_PROTECT(MySQLgroups = NEW_CHARACTER((Sint) 1)) ;
   SET_CHR_EL(MySQLgroups,0,C_S_CPY(""));
 
+  MEM_PROTECT(s_mysql_default_file = NEW_CHARACTER((Sint) 1)) ;
+  SET_CHR_EL(s_mysql_default_file,0,C_S_CPY(""));
+
   mgrHandle = RS_DBI_asMgrHandle(MGR_ID(conHandle));
   
   /* Connection parameters need to be put into a 7-element character
@@ -147,7 +157,8 @@ RS_MySQL_cloneConnection(Con_Handle *conHandle)
   
   MEM_UNPROTECT(2); 
 
-  return RS_MySQL_newConnection(mgrHandle, con_params, MySQLgroups);
+  return RS_MySQL_newConnection(mgrHandle, con_params, MySQLgroups,
+             s_mysql_default_file);
 }
 
 RS_MySQL_conParams *
@@ -179,7 +190,7 @@ RS_MySQL_freeConParams(RS_MySQL_conParams *conParams)
 
 Con_Handle *
 RS_MySQL_newConnection(Mgr_Handle *mgrHandle, s_object *con_params, 
-		       s_object *MySQLgroups)
+		       s_object *MySQLgroups, s_object *s_mysql_default_file)
 {
   S_EVALUATOR
 
@@ -192,7 +203,7 @@ RS_MySQL_newConnection(Mgr_Handle *mgrHandle, s_object *con_params,
   char      *unix_socket = NULL;
   int       i; 
   Sint      ngroups;
-  char      **groups;
+  char      **groups, *mysql_default_file;
 #if HAVE_GETOPT_LONG
   int   argc, option_index;    /* TODO: What about Mac OS, OS X??     */
   char  **argv;                /* we do have MySQL's load_defaults() */
@@ -218,7 +229,7 @@ RS_MySQL_newConnection(Mgr_Handle *mgrHandle, s_object *con_params,
    * [rs-dbi] sections of the MySQL configuration files.  We
    * recognize options in the [client] and [rs-dbi] sections, plus any
    * other passed from S/R in the MySQLgroups character vector.  
-   * Note that we're faking the argc and argv -- it just simpler (and the
+   * Note that we're faking the argc and argv -- it's just simpler (and the
    * recommended way from MySQL perspective).  Re-initialize the
    * getopt_long buffers by setting optind = 0 (defined in getopt.h),
    * this is needed to avoid getopt_long "remembering" options from
@@ -226,6 +237,10 @@ RS_MySQL_newConnection(Mgr_Handle *mgrHandle, s_object *con_params,
    * TODO: This can't be thread-safe, can it?
    *
    */
+  if(GET_LENGTH(s_mysql_default_file)==1){
+      mysql_default_file = RS_DBI_copyString(CHR_EL(s_mysql_default_file,0));
+      mysql_options(my_connection, MYSQL_READ_DEFAULT_FILE, mysql_default_file);
+  }
   ngroups = GET_LENGTH(MySQLgroups);
   groups = (char **) S_alloc((long) ngroups+3, (int) sizeof(char **));
   groups[0] = RS_DBI_copyString("client");  
@@ -1531,3 +1546,37 @@ add_group(s_object *group_names, s_object *data,
    SET_CHR_EL(group_names, ngroup, C_S_CPY(buff));
    return;
 }
+
+/* the following function was kindly provided by Mikhail Kondrin 
+ * it returns the last inserted index. 
+ * TODO: It returns an int, but it can potentially be inadequate
+ *       if the index is anunsigned integer.  Should we return
+ *       a numeric instead?
+ */
+s_object *
+RS_MySQL_insertid(Con_Handle *conHandle)
+{
+  S_EVALUATOR
+ 
+  MYSQL   *my_con;
+  RS_DBI_connection  *con;
+  s_object   *output;
+  char *conDesc[] = {"iid"};
+  Stype conType[] = {INTEGER_TYPE};    /* dj: are we sure an int will do? */
+  Sint  conLen[]  = {1};
+
+  con = RS_DBI_getConnection(conHandle);
+  my_con = (MYSQL *) con->drvConnection;
+  output = RS_DBI_createNamedList(conDesc, conType, conLen, 1);
+#ifndef USING_R
+  if(IS_LIST(output))
+    output = AS_LIST(output);
+  else
+    RS_DBI_errorMessage("internal error: could not alloc named list",
+                        RS_DBI_ERROR);
+#endif
+  LST_INT_EL(output,0,0) = (Sint) mysql_insert_id(my_con);
+
+  return output;
+
+}
